<?php
    /**
     * @class  cache
     * @author NHN (developers@xpressengine.com)
     * @brief  cache모듈의 high class
     **/

    class cache extends ModuleObject {

        /**
         * @brief 설치시 추가 작업이 필요할시 구현
         **/
        function moduleInstall() {
            // action forward에 등록 (관리자 모드에서 사용하기 위함)
            $oModuleController = &getController('module');

            // 캐시 정보를 기록할 디렉토리 생성
            FileHandler::makeDir('./files/member_extra_info/cache');

            $oModuleController = &getController('module');

            // 최고레벨
            $config->max_level = 30;

            // 레벨별 점수
            for($i=1;$i<=30;$i++) {
                $config->level_step[$i] = pow($i,2)*90;
            }

            // 회원가입
            $config->signup_cache = 0;

            // 로그인 가입
            $config->login_cache = 0;

            // 캐시 호칭
            $config->cache_name = 'cache';

            // 레벨 아이콘 디렉토리
            $config->level_icon = "default";

            // 점수가 없을때 다운로드 금지 기능
            $config->disable_download = true;

            /**
             * 모듈별 기본 점수 및 각 action 정의 (게시판,블로그외에 어떤 모듈이 생길지 모르니 act값을 명시한다
             **/

            // 글작성
            $config->insert_document = 0;

            $config->insert_document_act = 'procBoardInsertDocument';
            $config->delete_document_act = 'procBoardDeleteDocument';

            // 댓글작성
            $config->insert_comment = 0;

            $config->insert_comment_act = 'procBoardInsertComment,procBlogInsertComment';
            $config->delete_comment_act = 'procBoardDeleteComment,procBlogDeleteComment';

            // 업로드
            $config->upload_file = 0;

            $config->upload_file_act = 'procFileUpload';
            $config->delete_file_act = 'procFileDelete';

            // 다운로드
            $config->download_file = 0;
            $config->download_file_act = 'procFileDownload';

            // 조회
            $config->read_document = 0;

            // 추천 / 비추천
            $config->voted = 0;
            $config->blamed = 0;

            // 설정 저장
            $oModuleController->insertModuleConfig('cache', $config);

            // 빠른 실행을 위해서 act list를 캐싱
            $oCacheController = &getAdminController('cache');
            $oCacheController->cacheActList();

            // 가입/글작성/댓글작성/파일업로드/다운로드에 대한 트리거 추가
            $oModuleController->insertTrigger('member.insertMember', 'cache', 'controller', 'triggerInsertMember', 'after');
            $oModuleController->insertTrigger('document.insertDocument', 'cache', 'controller', 'triggerInsertDocument', 'after');
            $oModuleController->insertTrigger('document.deleteDocument', 'cache', 'controller', 'triggerBeforeDeleteDocument', 'before');
            $oModuleController->insertTrigger('document.deleteDocument', 'cache', 'controller', 'triggerDeleteDocument', 'after');
            $oModuleController->insertTrigger('comment.insertComment', 'cache', 'controller', 'triggerInsertComment', 'after');
            $oModuleController->insertTrigger('comment.deleteComment', 'cache', 'controller', 'triggerDeleteComment', 'after');
            $oModuleController->insertTrigger('file.insertFile', 'cache', 'controller', 'triggerInsertFile', 'after');
            $oModuleController->insertTrigger('file.deleteFile', 'cache', 'controller', 'triggerDeleteFile', 'after');
            $oModuleController->insertTrigger('file.downloadFile', 'cache', 'controller', 'triggerBeforeDownloadFile', 'before');
            $oModuleController->insertTrigger('file.downloadFile', 'cache', 'controller', 'triggerDownloadFile', 'after');
            $oModuleController->insertTrigger('member.doLogin', 'cache', 'controller', 'triggerAfterLogin', 'after');
            $oModuleController->insertTrigger('module.dispAdditionSetup', 'cache', 'view', 'triggerDispCacheAdditionSetup', 'after');
            $oModuleController->insertTrigger('document.updateReadedCount', 'cache', 'controller', 'triggerUpdateReadedCount', 'after');

            // 추천 / 비추천에 대한 트리거 추가 2008.05.13 haneul
            $oModuleController->insertTrigger('document.updateVotedCount', 'cache', 'controller', 'triggerUpdateVotedCount', 'after');

            // 임시저장글을 정상 저장시 캐시 지급하도록 트리거 추가 2009. 05. 19 zero
            $oModuleController->insertTrigger('document.updateDocument', 'cache', 'controller', 'triggerUpdateDocument', 'before');

            return new Object();
        }

        /**
         * @brief 설치가 이상이 없는지 체크하는 method
         **/
        function checkUpdate() {
            // cache 모듈 정보 가져옴
            $oModuleModel = &getModel('module');

            // 가입/글작성/댓글작성/파일업로드/다운로드에 대한 트리거 추가
            if(!$oModuleModel->getTrigger('member.insertMember', 'cache', 'controller', 'triggerInsertMember', 'after')) return true;
            if(!$oModuleModel->getTrigger('document.insertDocument', 'cache', 'controller', 'triggerInsertDocument', 'after')) return true;
            if(!$oModuleModel->getTrigger('document.deleteDocument', 'cache', 'controller', 'triggerBeforeDeleteDocument', 'before')) return true;
            if(!$oModuleModel->getTrigger('document.deleteDocument', 'cache', 'controller', 'triggerDeleteDocument', 'after')) return true;
            if(!$oModuleModel->getTrigger('comment.insertComment', 'cache', 'controller', 'triggerInsertComment', 'after')) return true;
            if(!$oModuleModel->getTrigger('comment.deleteComment', 'cache', 'controller', 'triggerDeleteComment', 'after')) return true;
            if(!$oModuleModel->getTrigger('file.insertFile', 'cache', 'controller', 'triggerInsertFile', 'after')) return true;
            if(!$oModuleModel->getTrigger('file.deleteFile', 'cache', 'controller', 'triggerDeleteFile', 'after')) return true;
            if(!$oModuleModel->getTrigger('file.downloadFile', 'cache', 'controller', 'triggerBeforeDownloadFile', 'before')) return true;
            if(!$oModuleModel->getTrigger('file.downloadFile', 'cache', 'controller', 'triggerDownloadFile', 'after')) return true;
            if(!$oModuleModel->getTrigger('member.doLogin', 'cache', 'controller', 'triggerAfterLogin', 'after')) return true;
            if(!$oModuleModel->getTrigger('module.dispAdditionSetup', 'cache', 'view', 'triggerDispCacheAdditionSetup', 'after')) return true;
            if(!$oModuleModel->getTrigger('document.updateReadedCount', 'cache', 'controller', 'triggerUpdateReadedCount', 'after')) return true;

            // 추천 / 비추천에 대한 트리거 추가 2008.05.13 haneul
            if(!$oModuleModel->getTrigger('document.updateVotedCount', 'cache', 'controller', 'triggerUpdateVotedCount', 'after')) return true;

            // 임시저장글을 정상 저장시 캐시 지급하도록 트리거 추가 2009. 05. 19 zero
            if(!$oModuleModel->getTrigger('document.updateDocument', 'cache', 'controller', 'triggerUpdateDocument', 'before')) return true;

            return false;
        }

        /**
         * @brief 업데이트 실행
         **/
        function moduleUpdate() {
            // cache 모듈 정보 가져옴
            $oModuleModel = &getModel('module');
            $oModuleController = &getController('module');

            // 가입/글작성/댓글작성/파일업로드/다운로드에 대한 트리거 추가
            if(!$oModuleModel->getTrigger('member.insertMember', 'cache', 'controller', 'triggerInsertMember', 'after')) 
                $oModuleController->insertTrigger('member.insertMember', 'cache', 'controller', 'triggerInsertMember', 'after');
            if(!$oModuleModel->getTrigger('document.insertDocument', 'cache', 'controller', 'triggerInsertDocument', 'after')) 
                $oModuleController->insertTrigger('document.insertDocument', 'cache', 'controller', 'triggerInsertDocument', 'after');
            if(!$oModuleModel->getTrigger('document.deleteDocument', 'cache', 'controller', 'triggerBeforeDeleteDocument', 'before')) 
                $oModuleController->insertTrigger('document.deleteDocument', 'cache', 'controller', 'triggerBeforeDeleteDocument', 'before');
            if(!$oModuleModel->getTrigger('document.deleteDocument', 'cache', 'controller', 'triggerDeleteDocument', 'after')) 
                $oModuleController->insertTrigger('document.deleteDocument', 'cache', 'controller', 'triggerDeleteDocument', 'after');
            if(!$oModuleModel->getTrigger('comment.insertComment', 'cache', 'controller', 'triggerInsertComment', 'after')) 
                $oModuleController->insertTrigger('comment.insertComment', 'cache', 'controller', 'triggerInsertComment', 'after');
            if(!$oModuleModel->getTrigger('comment.deleteComment', 'cache', 'controller', 'triggerDeleteComment', 'after')) 
                $oModuleController->insertTrigger('comment.deleteComment', 'cache', 'controller', 'triggerDeleteComment', 'after');
            if(!$oModuleModel->getTrigger('file.insertFile', 'cache', 'controller', 'triggerInsertFile', 'after')) 
                $oModuleController->insertTrigger('file.insertFile', 'cache', 'controller', 'triggerInsertFile', 'after');
            if(!$oModuleModel->getTrigger('file.deleteFile', 'cache', 'controller', 'triggerDeleteFile', 'after')) 
                $oModuleController->insertTrigger('file.deleteFile', 'cache', 'controller', 'triggerDeleteFile', 'after');
            if(!$oModuleModel->getTrigger('file.downloadFile', 'cache', 'controller', 'triggerBeforeDownloadFile', 'before')) 
                $oModuleController->insertTrigger('file.downloadFile', 'cache', 'controller', 'triggerBeforeDownloadFile', 'before');
            if(!$oModuleModel->getTrigger('file.downloadFile', 'cache', 'controller', 'triggerDownloadFile', 'after')) 
                $oModuleController->insertTrigger('file.downloadFile', 'cache', 'controller', 'triggerDownloadFile', 'after');
            if(!$oModuleModel->getTrigger('member.doLogin', 'cache', 'controller', 'triggerAfterLogin', 'after'))
                $oModuleController->insertTrigger('member.doLogin', 'cache', 'controller', 'triggerAfterLogin', 'after');
            if(!$oModuleModel->getTrigger('module.dispAdditionSetup', 'cache', 'view', 'triggerDispCacheAdditionSetup', 'after')) 
                $oModuleController->insertTrigger('module.dispAdditionSetup', 'cache', 'view', 'triggerDispCacheAdditionSetup', 'after');
            if(!$oModuleModel->getTrigger('document.updateReadedCount', 'cache', 'controller', 'triggerUpdateReadedCount', 'after')) 
                $oModuleController->insertTrigger('document.updateReadedCount', 'cache', 'controller', 'triggerUpdateReadedCount', 'after');

            // 추천 / 비추천에 대한 트리거 추가 2008.05.13 haneul
            if(!$oModuleModel->getTrigger('document.updateVotedCount', 'cache', 'controller', 'triggerUpdateVotedCount', 'after'))
                $oModuleController->insertTrigger('document.updateVotedCount', 'cache', 'controller', 'triggerUpdateVotedCount', 'after');

            // 임시저장글을 정상 저장시 캐시 지급하도록 트리거 추가 2009. 05. 19 zero
            if(!$oModuleModel->getTrigger('document.updateDocument', 'cache', 'controller', 'triggerUpdateDocument', 'before')) 
                $oModuleController->insertTrigger('document.updateDocument', 'cache', 'controller', 'triggerUpdateDocument', 'before');

            return new Object(0, 'success_updated');
        }

        /**
         * @brief 캐시 파일 재생성
         **/
        function recompileCache() {
            // cache action 파일 재정의
            $oCacheAdminController = &getAdminController('cache');
            $oCacheAdminController->cacheActList();

        }
    }
?>
