<?php
    /**
     * @class  cacheModel
     * @author NHN (developers@xpressengine.com)
     * @brief  cache 모듈의 model class
     **/

    class cacheModel extends cache {

        /**
         * @brief 초기화
         **/
        function init() {
        }

        /**
         * @brief 캐시 정보가 있는지 체크
         **/
        function isExistsCache($member_srl) {
            $member_srl = abs($member_srl);
            $args->member_srl = $member_srl;
            $output = executeQuery('cache.getCache', $args);
            if($output->data->member_srl == $member_srl) return true;
            return false;
        }

        /**
         * @brief 캐시를 구해옴
         **/
        function getCache($member_srl, $from_db = false) {
            $member_srl = abs($member_srl);
            $path = sprintf('./files/member_extra_info/cache/%s',getNumberingPath($member_srl));
            if(!is_dir($path)) FileHandler::makeDir($path);
            $cache_filename = sprintf('%s%d.cache.txt', $path, $member_srl);

            if(!$from_db && file_exists($cache_filename)) return trim(FileHandler::readFile($cache_filename));

            // DB에서 가져옴
            $args->member_srl = $member_srl;
            $output = executeQuery('cache.getCache', $args);
            $cache = (int)$output->data->cache;

            FileHandler::writeFile($cache_filename, $cache);

            return $cache;
        }

        /**
         * @brief 레벨을 구함
         **/
        function getLevel($cache, $level_step) {
            $level_count = count($level_step);
            for($level=0;$level<=$level_count;$level++) if($cache < $level_step[$level]) break;
            $level --;
            return $level;
        }

		function getMembersCacheInfo()
		{
			$member_srls = Context::get('member_srls');
			$member_srls = explode(',',$member_srls);
			if(count($member_srls)==0) return;
			array_unique($member_srls);

			$oModuleModel = &getModel('module');
			$config = $oModuleModel->getModuleConfig('cache');

			$info = array();
			foreach($member_srls as $v)
			{
				$obj = new stdClass;
				$obj->cache = $this->getCache($v);
				$obj->level = $this->getLevel($obj->cache, $config->level_step);
				$obj->member_srl = $v;
				$info[] = $obj;
			}

			$this->add('cache_info',$info);
		}


        /**
         * @brief 캐시 순 회원목록 가져오기
         **/
        function getMemberList($args = null) {

            // 검색 옵션 정리
            $args->is_admin = Context::get('is_admin')=='Y'?'Y':'';
            $args->is_denied = Context::get('is_denied')=='Y'?'Y':'';
            $args->selected_group_srl = Context::get('selected_group_srl');

            $search_target = trim(Context::get('search_target'));
            $search_keyword = trim(Context::get('search_keyword'));

            if($search_target && $search_keyword) {
                switch($search_target) {
                    case 'user_id' :
                            if($search_keyword) $search_keyword = str_replace(' ','%',$search_keyword);
                            $args->s_user_id = $search_keyword;
                        break;
                    case 'user_name' :
                            if($search_keyword) $search_keyword = str_replace(' ','%',$search_keyword);
                            $args->s_user_name = $search_keyword;
                        break;
                    case 'nick_name' :
                            if($search_keyword) $search_keyword = str_replace(' ','%',$search_keyword);
                            $args->s_nick_name = $search_keyword;
                        break;
                    case 'email_address' :
                            if($search_keyword) $search_keyword = str_replace(' ','%',$search_keyword);
                            $args->s_email_address = $search_keyword;
                        break;
                    case 'regdate' :
                            $args->s_regdate = $search_keyword;
                        break;
                    case 'last_login' :
                            $args->s_last_login = $search_keyword;
                        break;
                    case 'extra_vars' :
                            $args->s_extra_vars = $search_keyword;
                        break;
                }
            }

            // selected_group_srl이 있으면 query id를 변경 (table join때문에)
            if($args->selected_group_srl) {
                $query_id = 'cache.getMemberListWithinGroup';
            } else {
                $query_id = 'cache.getMemberList';
            }

            $output = executeQuery($query_id, $args);

            if($output->total_count) {
                $oModuleModel = &getModel('module');
                $config = $oModuleModel->getModuleConfig('cache');

                foreach($output->data as $key => $val) {
                    $output->data[$key]->level = $this->getLevel($val->cache, $config->level_step);
                }
            }

            return $output;
        }
    }
?>
